
#ifndef PLS_TASK_H
#define PLS_TASK_H

#include "pls/internal/scheduling/task_manager.h"
#include "pls/internal/scheduling/thread_state.h"

namespace pls {
namespace internal {
namespace scheduling {

/**
 * A task to be executed by the runtime system.
 * Tasks are guaranteed to be executed exactly once.
 *
 * Override the execute_internal() method for your custom code.
 */
class task {
  friend class scheduler;

  // TODO: Add ref to continuation
  task_manager::task_manager_state task_manager_state_;

 protected:
  explicit task();

  /**
   * Allow to allocate extra memory during run-time for this task.
   * Memory will be pushed onto the stack (in aligned memory, thus avoid many small chunks).
   *
   * Memory is fully self managed. Calling e.g. de-constructors when not needing objects
   * anymore is the users responsibility (memory is simply re-used after the life time of the task ends).
   *
   * @param size Number of bytes to be allocated
   * @return The allocated memory region
   */
  void *allocate_memory(long size);

  /**
   * Overwrite this with the actual behaviour of concrete tasks.
   */
  virtual void execute_internal() = 0;

 private:
  void execute();
};

}
}
}

#endif //PLS_TASK_H
